@@ -3,7 +3,7 @@
  * @brief Main MongoDB Declarations
  */
 
-/*    Copyright 2009-2011 10gen Inc.
+/*    Copyright 2009-2012 10gen Inc.
  *
  *    Licensed under the Apache License, Version 2.0 (the "License");
  *    you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@
 MONGO_EXTERN_C_START
 
 #define MONGO_MAJOR 0
-#define MONGO_MINOR 4
+#define MONGO_MINOR 7
 #define MONGO_PATCH 0
 
 #define MONGO_OK 0
@@ -34,6 +34,10 @@ MONGO_EXTERN_C_START
 
 #define MONGO_DEFAULT_PORT 27017
 
+#define MONGO_DEFAULT_MAX_BSON_SIZE 4 * 1024 * 1024
+
+#define MONGO_ERR_LEN 128
+
 typedef enum mongo_error_t {
     MONGO_CONN_SUCCESS = 0,  /**< Connection success! */
     MONGO_CONN_NO_SOCKET,    /**< Could not create a socket. */
@@ -44,10 +48,15 @@ typedef enum mongo_error_t {
     MONGO_CONN_NO_PRIMARY,   /**< Can't find primary in replica set. Connection closed. */
 
     MONGO_IO_ERROR,          /**< An error occurred while reading or writing on the socket. */
+    MONGO_SOCKET_ERROR,      /**< Other socket error. */
     MONGO_READ_SIZE_ERROR,   /**< The response is not the expected length. */
     MONGO_COMMAND_FAILED,    /**< The command returned with 'ok' value of 0. */
+    MONGO_WRITE_ERROR,       /**< Write with given write_concern returned an error. */
+    MONGO_NS_INVALID,        /**< The name for the ns (database or collection) is invalid. */
     MONGO_BSON_INVALID,      /**< BSON not valid for the specified op. */
-    MONGO_BSON_NOT_FINISHED  /**< BSON object has not been finished. */
+    MONGO_BSON_NOT_FINISHED, /**< BSON object has not been finished. */
+    MONGO_BSON_TOO_LARGE,    /**< BSON object exceeds max BSON size. */
+    MONGO_WRITE_CONCERN_INVALID /**< Supplied write concern object is invalid. */
 } mongo_error_t;
 
 typedef enum mongo_cursor_error_t {
@@ -78,6 +87,10 @@ enum mongo_update_opts {
     MONGO_UPDATE_BASIC = 0x4
 };
 
+enum mongo_insert_opts {
+    MONGO_CONTINUE_ON_ERROR = 0x1
+};
+
 enum mongo_cursor_opts {
     MONGO_TAILABLE = ( 1<<1 ),        /**< Create a tailable cursor. */
     MONGO_SLAVE_OK = ( 1<<2 ),        /**< Allow queries on a non-primary node. */
@@ -130,26 +143,39 @@ typedef struct mongo_host_port {
     struct mongo_host_port *next;
 } mongo_host_port;
 
+typedef struct mongo_write_concern {
+    int w;            /**< Number of total replica write copies to complete including the primary. */
+    int wtimeout;     /**< Number of milliseconds before replication timeout. */
+    int j;            /**< If non-zero, block until the journal sync. */
+    int fsync;        /**< Same a j with journaling enabled; otherwise, call fsync. */
+    const char *mode; /**< Either "majority" or a getlasterrormode. Overrides w value. */
+
+    bson *cmd; /**< The BSON object representing the getlasterror command. */
+} mongo_write_concern;
+
 typedef struct {
     mongo_host_port *seeds;        /**< List of seeds provided by the user. */
     mongo_host_port *hosts;        /**< List of host/ports given by the replica set */
     char *name;                    /**< Name of the replica set. */
     bson_bool_t primary_connected; /**< Primary node connection status. */
-} mongo_replset;
+} mongo_replica_set;
 
 typedef struct mongo {
     mongo_host_port *primary;  /**< Primary connection info. */
-    mongo_replset *replset;    /**< replset object if connected to a replica set. */
+    mongo_replica_set *replica_set;    /**< replica_set object if connected to a replica set. */
     int sock;                  /**< Socket file descriptor. */
     int flags;                 /**< Flags on this connection object. */
     int conn_timeout_ms;       /**< Connection timeout in milliseconds. */
     int op_timeout_ms;         /**< Read and write timeout in milliseconds. */
+    int max_bson_size;         /**< Largest BSON object allowed on this connection. */
     bson_bool_t connected;     /**< Connection status. */
+    mongo_write_concern *write_concern; /**< The default write concern. */
 
-    mongo_error_t err;         /**< Most recent driver error code. */
-    char *errstr;              /**< String version of most recent driver error code. */
-    int lasterrcode;           /**< getlasterror given by the server on calls. */
-    char *lasterrstr;          /**< getlasterror string generated by server. */
+    mongo_error_t err;          /**< Most recent driver error code. */
+    int errcode;                /**< Most recent errno or WSAGetLastError(). */
+    char errstr[MONGO_ERR_LEN]; /**< String version of error. */
+    int lasterrcode;            /**< getlasterror code from the server. */
+    char lasterrstr[MONGO_ERR_LEN]; /**< getlasterror string from the server. */
 } mongo;
 
 typedef struct {
@@ -160,28 +186,20 @@ typedef struct {
     int seen;          /**< Number returned so far. */
     bson current;      /**< This cursor's current bson object. */
     mongo_cursor_error_t err; /**< Errors on this cursor. */
-    bson *query;       /**< Bitfield containing cursor options. */
-    bson *fields;      /**< Bitfield containing cursor options. */
+    const bson *query; /**< Bitfield containing cursor options. */
+    const bson *fields;/**< Bitfield containing cursor options. */
     int options;       /**< Bitfield containing cursor options. */
     int limit;         /**< Bitfield containing cursor options. */
     int skip;          /**< Bitfield containing cursor options. */
 } mongo_cursor;
 
-/* Connection API */
+/*********************************************************************
+Connection API
+**********************************************************************/
 
-MONGO_EXPORT mongo* mongo_create();
-MONGO_EXPORT void mongo_dispose(mongo* conn);
-MONGO_EXPORT int mongo_get_err(mongo* conn);
-MONGO_EXPORT int mongo_is_connected(mongo* conn);
-MONGO_EXPORT int mongo_get_op_timeout(mongo* conn);
-MONGO_EXPORT const char* mongo_get_primary(mongo* conn);
-MONGO_EXPORT int mongo_get_socket(mongo* conn) ;
-MONGO_EXPORT int mongo_get_host_count(mongo* conn);
-MONGO_EXPORT const char* mongo_get_host(mongo* conn, int i);
-MONGO_EXPORT mongo_cursor* mongo_cursor_create();
-MONGO_EXPORT void mongo_cursor_dispose(mongo_cursor* cursor);
-MONGO_EXPORT int  mongo_get_server_err(mongo* conn);
-MONGO_EXPORT const char*  mongo_get_server_err_string(mongo* conn);
+/** Initialize sockets for Windows.
+ */
+MONGO_EXPORT void mongo_init_sockets( void );
 
 /**
  * Initialize a new mongo connection object. You must initialize each mongo
@@ -193,7 +211,7 @@ MONGO_EXPORT const char*  mongo_get_server_err_string(mongo* conn);
  *  @param conn a mongo connection object allocated on the stack
  *      or heap.
  */
-void mongo_init( mongo *conn );
+MONGO_EXPORT void mongo_init( mongo *conn );
 
 /**
  * Connect to a single MongoDB server.
@@ -205,9 +223,32 @@ void mongo_init( mongo *conn );
  * @return MONGO_OK or MONGO_ERROR on failure. On failure, a constant of type
  *   mongo_error_t will be set on the conn->err field.
  */
+MONGO_EXPORT int mongo_client( mongo *conn , const char *host, int port );
+
+/**
+ * DEPRECATED - use mongo_client.
+ * Connect to a single MongoDB server.
+ *
+ * @param conn a mongo object.
+ * @param host a numerical network address or a network hostname.
+ * @param port the port to connect to.
+ *
+ * @return MONGO_OK or MONGO_ERROR on failure. On failure, a constant of type
+ *   mongo_error_t will be set on the conn->err field.
+ */
 MONGO_EXPORT int mongo_connect( mongo *conn , const char *host, int port );
 
 /**
+ * Set up this connection object for connecting to a replica set.
+ * To connect, pass the object to mongo_replica_set_connect().
+ *
+ * @param conn a mongo object.
+ * @param name the name of the replica set to connect to.
+ * */
+MONGO_EXPORT void mongo_replica_set_init( mongo *conn, const char *name );
+
+/**
+ * DEPRECATED - use mongo_replica_set_init.
  * Set up this connection object for connecting to a replica set.
  * To connect, pass the object to mongo_replset_connect().
  *
@@ -225,6 +266,18 @@ MONGO_EXPORT void mongo_replset_init( mongo *conn, const char *name );
  * @param host a numerical network address or a network hostname.
  * @param port the port to connect to.
  */
+MONGO_EXPORT void mongo_replica_set_add_seed( mongo *conn, const char *host, int port );
+
+/**
+ * DEPRECATED - use mongo_replica_set_add_seed.
+ * Add a seed node to the replica set connection object.
+ *
+ * You must specify at least one seed node before connecting to a replica set.
+ *
+ * @param conn a mongo object.
+ * @param host a numerical network address or a network hostname.
+ * @param port the port to connect to.
+ */
 MONGO_EXPORT void mongo_replset_add_seed( mongo *conn, const char *host, int port );
 
 /**
@@ -237,6 +290,31 @@ MONGO_EXPORT void mongo_replset_add_seed( mongo *conn, const char *host, int por
 void mongo_parse_host( const char *host_string, mongo_host_port *host_port );
 
 /**
+ * Utility function for validation database and collection names.
+ *
+ * @param conn a mongo object.
+ *
+ * @return MONGO_OK or MONGO_ERROR on failure. On failure, a constant of type
+ *   mongo_conn_return_t will be set on the conn->err field.
+ *
+ */
+MONGO_EXPORT int mongo_validate_ns( mongo *conn, const char *ns );
+
+/**
+ * Connect to a replica set.
+ *
+ * Before passing a connection object to this function, you must already have called
+ * mongo_set_replica_set and mongo_replica_set_add_seed.
+ *
+ * @param conn a mongo object.
+ *
+ * @return MONGO_OK or MONGO_ERROR on failure. On failure, a constant of type
+ *   mongo_conn_return_t will be set on the conn->err field.
+ */
+MONGO_EXPORT int mongo_replica_set_client( mongo *conn );
+
+/**
+ * DEPRECATED - use mongo_replica_set_client.
  * Connect to a replica set.
  *
  * Before passing a connection object to this function, you must already have called
@@ -303,61 +381,130 @@ MONGO_EXPORT void mongo_disconnect( mongo *conn );
  */
 MONGO_EXPORT void mongo_destroy( mongo *conn );
 
+/**
+ * Specify the write concern object that this connection should use
+ * by default for all writes (inserts, updates, and deletes). This value
+ * can be overridden by passing a write_concern object to any write function.
+ *
+ * @param conn a mongo object.
+ * @param write_concern pointer to a write concern object.
+ *
+ */
+MONGO_EXPORT void mongo_set_write_concern( mongo *conn,
+        mongo_write_concern *write_concern );
+
+
+/*********************************************************************
+CRUD API
+**********************************************************************/
+
 /**
  * Insert a BSON document into a MongoDB server. This function
  * will fail if the supplied BSON struct is not UTF-8 or if
  * the keys are invalid for insert (contain '.' or start with '$').
  *
+ * The default write concern set on the conn object will be used.
+ *
  * @param conn a mongo object.
  * @param ns the namespace.
  * @param data the bson data.
+ * @param custom_write_concern a write concern object that will
+ *     override any write concern set on the conn object.
  *
  * @return MONGO_OK or MONGO_ERROR. If the conn->err
  *     field is MONGO_BSON_INVALID, check the err field
  *     on the bson struct for the reason.
  */
-MONGO_EXPORT int mongo_insert( mongo *conn, const char *ns, bson *data );
+MONGO_EXPORT int mongo_insert( mongo *conn, const char *ns, const bson *data,
+                               mongo_write_concern *custom_write_concern );
 
 /**
  * Insert a batch of BSON documents into a MongoDB server. This function
  * will fail if any of the documents to be inserted is invalid.
  *
+ * The default write concern set on the conn object will be used.
+ *
  * @param conn a mongo object.
  * @param ns the namespace.
  * @param data the bson data.
  * @param num the number of documents in data.
+ * @param custom_write_concern a write concern object that will
+ *     override any write concern set on the conn object.
+ * @param flags flags on this batch insert. Currently, this value
+ *     may be 0 or MONGO_CONTINUE_ON_ERROR, which will cause the
+ *     batch insert to continue even if a given insert in the batch fails.
  *
  * @return MONGO_OK or MONGO_ERROR.
  *
  */
-MONGO_EXPORT int mongo_insert_batch( mongo *conn , const char *ns ,
-                                     bson **data , int num );
+MONGO_EXPORT int mongo_insert_batch( mongo *conn, const char *ns,
+                                     const bson **data, int num, mongo_write_concern *custom_write_concern,
+                                     int flags );
 
 /**
  * Update a document in a MongoDB server.
  *
+ * The default write concern set on the conn object will be used.
+ *
  * @param conn a mongo object.
  * @param ns the namespace.
  * @param cond the bson update query.
  * @param op the bson update data.
  * @param flags flags for the update.
+ * @param custom_write_concern a write concern object that will
+ *     override any write concern set on the conn object.
  *
  * @return MONGO_OK or MONGO_ERROR with error stored in conn object.
  *
  */
 MONGO_EXPORT int mongo_update( mongo *conn, const char *ns, const bson *cond,
-                               const bson *op, int flags );
+                               const bson *op, int flags, mongo_write_concern *custom_write_concern );
 
 /**
  * Remove a document from a MongoDB server.
  *
+ * The default write concern set on the conn object will be used.
+ *
  * @param conn a mongo object.
  * @param ns the namespace.
  * @param cond the bson query.
+ * @param custom_write_concern a write concern object that will
+ *     override any write concern set on the conn object.
  *
  * @return MONGO_OK or MONGO_ERROR with error stored in conn object.
  */
-MONGO_EXPORT int mongo_remove( mongo *conn, const char *ns, const bson *cond );
+MONGO_EXPORT int mongo_remove( mongo *conn, const char *ns, const bson *cond,
+                               mongo_write_concern *custom_write_concern );
+
+
+/*********************************************************************
+Write Concern API
+**********************************************************************/
+
+/**
+ * Initialize a mongo_write_concern object. Effectively zeroes out the struct.
+ *
+ */
+MONGO_EXPORT void mongo_write_concern_init( mongo_write_concern *write_concern );
+
+/**
+ * Finish this write concern object by serializing the literal getlasterror
+ * command that will be sent to the server.
+ *
+ * You must call mongo_write_concern_destroy() to free the serialized BSON.
+ *
+ */
+MONGO_EXPORT int mongo_write_concern_finish( mongo_write_concern *write_concern );
+
+/**
+ * Free the write_concern object (specifically, the BSON that it owns).
+ *
+ */
+MONGO_EXPORT void mongo_write_concern_destroy( mongo_write_concern *write_concern );
+
+/*********************************************************************
+Cursor API
+**********************************************************************/
 
 /**
  * Find documents in a MongoDB server.
@@ -374,8 +521,8 @@ MONGO_EXPORT int mongo_remove( mongo *conn, const char *ns, const bson *cond );
  *     an error has occurred. For finer-grained error checking,
  *     use the cursor builder API instead.
  */
-MONGO_EXPORT mongo_cursor *mongo_find( mongo *conn, const char *ns, bson *query,
-                                       bson *fields, int limit, int skip, int options );
+MONGO_EXPORT mongo_cursor *mongo_find( mongo *conn, const char *ns, const bson *query,
+                                       const bson *fields, int limit, int skip, int options );
 
 /**
  * Initalize a new cursor object.
@@ -384,7 +531,7 @@ MONGO_EXPORT mongo_cursor *mongo_find( mongo *conn, const char *ns, bson *query,
  * @param ns the namespace, represented as the the database
  *     name and collection name separated by a dot. e.g., "test.users"
  */
-void mongo_cursor_init( mongo_cursor *cursor, mongo *conn, const char *ns );
+MONGO_EXPORT void mongo_cursor_init( mongo_cursor *cursor, mongo *conn, const char *ns );
 
 /**
  * Set the bson object specifying this cursor's query spec. If
@@ -397,7 +544,7 @@ void mongo_cursor_init( mongo_cursor *cursor, mongo *conn, const char *ns );
  *   $query, $orderby, $hint, and/or $explain. See
  *   http://www.mongodb.org/display/DOCS/Mongo+Wire+Protocol for details.
  */
-void mongo_cursor_set_query( mongo_cursor *cursor, bson *query );
+MONGO_EXPORT void mongo_cursor_set_query( mongo_cursor *cursor, const bson *query );
 
 /**
  * Set the fields to return for this cursor. If you want to return
@@ -407,7 +554,7 @@ void mongo_cursor_set_query( mongo_cursor *cursor, bson *query );
  * @param fields a bson object representing the fields to return.
  *   See http://www.mongodb.org/display/DOCS/Retrieving+a+Subset+of+Fields.
  */
-void mongo_cursor_set_fields( mongo_cursor *cursor, bson *fields );
+MONGO_EXPORT void mongo_cursor_set_fields( mongo_cursor *cursor, const bson *fields );
 
 /**
  * Set the number of documents to skip.
@@ -415,7 +562,7 @@ void mongo_cursor_set_fields( mongo_cursor *cursor, bson *fields );
  * @param cursor
  * @param skip
  */
-void mongo_cursor_set_skip( mongo_cursor *cursor, int skip );
+MONGO_EXPORT void mongo_cursor_set_skip( mongo_cursor *cursor, int skip );
 
 /**
  * Set the number of documents to return.
@@ -423,7 +570,7 @@ void mongo_cursor_set_skip( mongo_cursor *cursor, int skip );
  * @param cursor
  * @param limit
  */
-void mongo_cursor_set_limit( mongo_cursor *cursor, int limit );
+MONGO_EXPORT void mongo_cursor_set_limit( mongo_cursor *cursor, int limit );
 
 /**
  * Set any of the available query options (e.g., MONGO_TAILABLE).
@@ -432,7 +579,7 @@ void mongo_cursor_set_limit( mongo_cursor *cursor, int limit );
  * @param options a bitfield storing query options. See
  *   mongo_cursor_bitfield_t for available constants.
  */
-void mongo_cursor_set_options( mongo_cursor *cursor, int options );
+MONGO_EXPORT void mongo_cursor_set_options( mongo_cursor *cursor, int options );
 
 /**
  * Return the current BSON object data as a const char*. This is useful
@@ -440,7 +587,7 @@ void mongo_cursor_set_options( mongo_cursor *cursor, int options );
  *
  * @param cursor
  */
-const char *mongo_cursor_data( mongo_cursor *cursor );
+MONGO_EXPORT const char *mongo_cursor_data( mongo_cursor *cursor );
 
 /**
  * Return the current BSON object data as a const char*. This is useful
@@ -483,10 +630,13 @@ MONGO_EXPORT int mongo_cursor_destroy( mongo_cursor *cursor );
  *
  */
 /* out can be NULL if you don't care about results. useful for commands */
-MONGO_EXPORT bson_bool_t mongo_find_one( mongo *conn, const char *ns, bson *query,
-        bson *fields, bson *out );
+MONGO_EXPORT int mongo_find_one( mongo *conn, const char *ns, const bson *query,
+                                 const bson *fields, bson *out );
 
-/* MongoDB Helper Functions */
+
+/*********************************************************************
+Command API and Helpers
+**********************************************************************/
 
 /**
  * Count the number of documents in a collection matching a query.
@@ -500,10 +650,10 @@ MONGO_EXPORT bson_bool_t mongo_find_one( mongo *conn, const char *ns, bson *quer
  *     MONGO_ERROR is returned.
  */
 MONGO_EXPORT double mongo_count( mongo *conn, const char *db, const char *coll,
-                                 bson *query );
+                                 const bson *query );
 
 /**
- * Create a compouned index.
+ * Create a compound index.
  *
  * @param conn a mongo object.
  * @param ns the namespace.
@@ -515,7 +665,23 @@ MONGO_EXPORT double mongo_count( mongo *conn, const char *db, const char *coll,
  *
  * @return MONGO_OK if index is created successfully; otherwise, MONGO_ERROR.
  */
-MONGO_EXPORT int mongo_create_index( mongo *conn, const char *ns, bson *key, int options, bson *out );
+MONGO_EXPORT int mongo_create_index( mongo *conn, const char *ns,
+                                     const bson *key, int options, bson *out );
+
+/**
+ * Create a capped collection.
+ *
+ * @param conn a mongo object.
+ * @param ns the namespace (e.g., "dbname.collectioname")
+ * @param size the size of the capped collection in bytes.
+ * @param max the max number of documents this collection is
+ *   allowed to contain. If zero, this argument will be ignored
+ *   and the server will use the collection's size to age document out.
+ *   If using this option, ensure that the total size can contain this
+ *   number of documents.
+ */
+MONGO_EXPORT int mongo_create_capped_collection( mongo *conn, const char *db,
+        const char *collection, int size, int max, bson *out );
 
 /**
  * Create an index with a single key.
@@ -528,11 +694,8 @@ MONGO_EXPORT int mongo_create_index( mongo *conn, const char *ns, bson *key, int
  *
  * @return true if the index was created.
  */
-bson_bool_t mongo_create_simple_index( mongo *conn, const char *ns, const char *field, int options, bson *out );
-
-/* ----------------------------
-   COMMANDS
-   ------------------------------ */
+MONGO_EXPORT bson_bool_t mongo_create_simple_index( mongo *conn, const char *ns,
+        const char *field, int options, bson *out );
 
 /**
  * Run a command on a MongoDB server.
@@ -544,7 +707,8 @@ bson_bool_t mongo_create_simple_index( mongo *conn, const char *ns, const char *
  *
  * @return MONGO_OK if the command ran without error.
  */
-MONGO_EXPORT bson_bool_t mongo_run_command( mongo *conn, const char *db, bson *command, bson *out );
+MONGO_EXPORT int mongo_run_command( mongo *conn, const char *db,
+                                    const bson *command, bson *out );
 
 /**
  * Run a command that accepts a simple string key and integer value.
@@ -558,8 +722,8 @@ MONGO_EXPORT bson_bool_t mongo_run_command( mongo *conn, const char *db, bson *c
  * @return MONGO_OK or an error code.
  *
  */
-int mongo_simple_int_command( mongo *conn, const char *db,
-                              const char *cmd, int arg, bson *out );
+MONGO_EXPORT int mongo_simple_int_command( mongo *conn, const char *db,
+        const char *cmd, int arg, bson *out );
 
 /**
  * Run a command that accepts a simple string key and value.
@@ -573,7 +737,8 @@ int mongo_simple_int_command( mongo *conn, const char *db,
  * @return true if the command ran without error.
  *
  */
-bson_bool_t mongo_simple_str_command( mongo *conn, const char *db, const char *cmd, const char *arg, bson *out );
+MONGO_EXPORT int mongo_simple_str_command( mongo *conn, const char *db,
+        const char *cmd, const char *arg, bson *out );
 
 /**
  * Drop a database.
@@ -595,7 +760,8 @@ MONGO_EXPORT int mongo_cmd_drop_db( mongo *conn, const char *db );
  *
  * @return true if the collection drop was successful.
  */
-MONGO_EXPORT bson_bool_t mongo_cmd_drop_collection( mongo *conn, const char *db, const char *collection, bson *out );
+MONGO_EXPORT int mongo_cmd_drop_collection( mongo *conn, const char *db,
+        const char *collection, bson *out );
 
 /**
  * Add a database user.
@@ -607,7 +773,8 @@ MONGO_EXPORT bson_bool_t mongo_cmd_drop_collection( mongo *conn, const char *db,
  *
  * @return MONGO_OK or MONGO_ERROR.
   */
-MONGO_EXPORT int mongo_cmd_add_user( mongo *conn, const char *db, const char *user, const char *pass );
+MONGO_EXPORT int mongo_cmd_add_user( mongo *conn, const char *db,
+                                     const char *user, const char *pass );
 
 /**
  * Authenticate a user.
@@ -619,7 +786,8 @@ MONGO_EXPORT int mongo_cmd_add_user( mongo *conn, const char *db, const char *us
  *
  * @return MONGO_OK on sucess and MONGO_ERROR on failure.
  */
-MONGO_EXPORT int mongo_cmd_authenticate( mongo *conn, const char *db, const char *user, const char *pass );
+MONGO_EXPORT int mongo_cmd_authenticate( mongo *conn, const char *db,
+        const char *user, const char *pass );
 
 /**
  * Check if the current server is a master.
@@ -662,9 +830,44 @@ MONGO_EXPORT int mongo_cmd_get_prev_error( mongo *conn, const char *db, bson *ou
  * @param conn a mongo object.
  * @param db the name of the database.
  */
-void mongo_cmd_reset_error( mongo *conn, const char *db );
+MONGO_EXPORT void mongo_cmd_reset_error( mongo *conn, const char *db );
 
 
+/*********************************************************************
+Utility API
+**********************************************************************/
+
+MONGO_EXPORT mongo* mongo_create( void );
+MONGO_EXPORT void mongo_dispose(mongo* conn);
+MONGO_EXPORT int mongo_get_err(mongo* conn);
+MONGO_EXPORT int mongo_is_connected(mongo* conn);
+MONGO_EXPORT int mongo_get_op_timeout(mongo* conn);
+MONGO_EXPORT const char* mongo_get_primary(mongo* conn);
+MONGO_EXPORT int mongo_get_socket(mongo* conn) ;
+MONGO_EXPORT int mongo_get_host_count(mongo* conn);
+MONGO_EXPORT const char* mongo_get_host(mongo* conn, int i);
+MONGO_EXPORT mongo_cursor* mongo_cursor_create( void );
+MONGO_EXPORT void mongo_cursor_dispose(mongo_cursor* cursor);
+MONGO_EXPORT int  mongo_get_server_err(mongo* conn);
+MONGO_EXPORT const char*  mongo_get_server_err_string(mongo* conn);
+
+/**
+ * Set an error on a mongo connection object. Mostly for internal use.
+ *
+ * @param conn a mongo connection object.
+ * @param err a driver error code of mongo_error_t.
+ * @param errstr a string version of the error.
+ * @param errorcode Currently errno or WSAGetLastError().
+ */
+MONGO_EXPORT void __mongo_set_error( mongo *conn, mongo_error_t err,
+                                     const char *errstr, int errorcode );
+/**
+ * Clear all errors stored on a mongo connection object.
+ *
+ * @param conn a mongo connection object.
+ */
+MONGO_EXPORT void mongo_clear_errors( mongo *conn );
+
 MONGO_EXTERN_C_END
 
 #endif
